#
# pyJanus (Python Bindings for Janus)
#
# Copyright (c) 2022 Alwin Wang
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

import pyJanus


def test_set_var_def_example_00():
    """
    This checks pyJanus against the Examples/SetVarDefExample.cpp program 00
    """
    xml_path = "Examples/SetVarDefExample.xml"
    janus = pyJanus.Janus(xml_path)

    input00 = janus.get_variabledef("input00")
    output00 = janus.get_variabledef("output00")

    input00.set_value(15)
    assert round(output00.get_value(), 0) == 19


def test_set_var_def_example_01():
    """
    This checks pyJanus against the Examples/SetVarDefExample.cpp program 01
    """
    xml_path = "Examples/SetVarDefExample.xml"
    janus = pyJanus.Janus(xml_path)

    incrementer = janus.get_variabledef("incrementer")
    output01 = janus.get_variabledef("output01")

    incrementer.get_value()
    assert round(output01.get_value(), 0) == 6

    incrementer.get_value()
    assert round(output01.get_value(), 0) == 9

    incrementer.get_value()
    assert round(output01.get_value(), 0) == 12


def test_set_var_def_example_02():
    """
    This checks pyJanus against the Examples/SetVarDefExample.cpp program 02
    """
    xml_path = "Examples/SetVarDefExample.xml"
    janus = pyJanus.Janus(xml_path)

    input02 = janus.get_variabledef("input02")
    output02 = janus.get_variabledef("output02")
    function02 = janus.get_variabledef("function02")

    input02.set_value(42)
    function02.get_value()
    assert round(output02.get_value(), 0) == 42


def test_set_var_def_example_03():
    """
    This checks pyJanus against the Examples/SetVarDefExample.cpp program 03
    """
    xml_path = "Examples/SetVarDefExample.xml"
    janus = pyJanus.Janus(xml_path)

    input03 = janus.get_variabledef("input03")
    output03_00 = janus.get_variabledef("output03_00")
    function03 = janus.get_variabledef("function03")

    input03.set_value(123.456)
    function03.get_value()
    assert round(output03_00.get_value(), 3) == 123.456


def test_set_var_def_example_04():
    """
    This checks pyJanus against the Examples/SetVarDefExample.cpp program 04
    """
    xml_path = "Examples/SetVarDefExample.xml"
    janus = pyJanus.Janus(xml_path)

    function04_00 = janus.get_variabledef("function04_00")
    function04_01 = janus.get_variabledef("function04_01")

    assert round(function04_00.get_value(), 2) == 56.78
    assert round(function04_01.get_value(), 2) == 56.78


def test_set_var_def_example_05():
    """
    This checks pyJanus against the Examples/SetVarDefExample.cpp program 05
    """
    xml_path = "Examples/SetVarDefExample.xml"
    janus = pyJanus.Janus(xml_path)

    output05_00 = janus.get_variabledef("output05_00")
    output05_01 = janus.get_variabledef("output05_01")

    assert round(output05_00.get_value(), 1) == 0.0
    assert round(output05_01.get_value(), 1) == 40.0


def test_set_var_def_example_06():
    """
    This checks pyJanus against the Examples/SetVarDefExample.cpp program 06
    """
    xml_path = "Examples/SetVarDefExample.xml"
    janus = pyJanus.Janus(xml_path)

    output06_00 = janus.get_variabledef("output06_00")
    output06_01 = janus.get_variabledef("output06_01")
    output06_02 = janus.get_variabledef("output06_02")
    output06_03 = janus.get_variabledef("output06_03")
    internal06_00 = janus.get_variabledef("internal06_00")
    internal06_01 = janus.get_variabledef("internal06_01")
    internal06_02 = janus.get_variabledef("internal06_02")
    internal06_21 = janus.get_variabledef("internal06_21")

    assert round(output06_00.get_value(), 1) == 2704.0
    assert round(internal06_00.get_value(), 1) == 0.0
    assert round(output06_01.get_value(), 1) == 52.0
    assert round(internal06_01.get_value(), 1) == 0.0
    assert round(output06_02.get_value(), 1) == 52.0
    assert round(internal06_02.get_value(), 1) == 0.0
    assert round(output06_03.get_value(), 1) == 231.0
    assert round(internal06_21.get_value(), 1) == 0.0



#
# pyJanus (Python Bindings for Janus)
#
# Copyright (c) 2022 Alwin Wang
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

import pyJanus


def test_janus_variable_manager_example_00():
    """
    This checks pyJanus against the Examples/SetVarDefExample.cpp program 00
    """
    xml_path = "Examples/JanusVariableManagerExample.xml"
    jvm = pyJanus.JanusVariableManager(xml_path)

    input00 = pyJanus.JanusVariable(
        "input00", pyJanus.janusInputVariable, pyJanus.janusMandatory, "m s-1", 0.0
    )
    output00 = pyJanus.JanusVariable(
        "output00", pyJanus.janusOutputVariable, pyJanus.janusMandatory, "kn", 0.0
    )
    jInput00 = jvm.push_back(input00)
    jOutput00 = jvm.push_back(output00)
    jv_input00 = jvm[jInput00]
    jv_output00 = jvm[jOutput00]

    assert jv_input00.is_available()
    assert jv_output00.is_available()

    assert round(jv_output00.get_value(), 0) == 0
    assert jv_input00.set_value(1.0)
    assert round(jv_output00.get_value(), 5) == 1.94384

    assert jvm[jInput00].set_value(0.0)
    assert round(jvm[jOutput00].get_value(), 0) == 0
    assert jvm[jInput00].set_value(1.0)
    assert round(jvm[jOutput00].get_value(), 5) == 1.94384


def test_janus_variable_manager_example_01():
    """
    This checks pyJanus against the Examples/SetVarDefExample.cpp program 01
    """
    xml_path = "Examples/JanusVariableManagerExample.xml"
    jvm = pyJanus.JanusVariableManager(xml_path)

    input00 = pyJanus.JanusVariable(
        "input00", pyJanus.janusInputVariable, pyJanus.janusMandatory, "m s-1", 0.0
    )
    output00 = pyJanus.JanusVariable(
        "output00", pyJanus.janusOutputVariable, pyJanus.janusMandatory, "kn", 0.0
    )
    jInput00 = jvm.push_back(input00)
    jOutput00 = jvm.push_back(output00)
    jv_input00 = jvm[jInput00]
    jv_output00 = jvm[jOutput00]

    assert round(jv_output00.get_value(), 0) == 0
    assert jv_input00.set_value(1.0)
    assert round(jv_output00.get_value(), 5) == 1.94384
    assert not jv_input00.set_value(1.0)
    assert jv_input00.set_value(2.0)
    assert round(jv_output00.get_value(), 5) == 3.88769



#
# pyJanus (Python Bindings for Janus)
#
# Copyright (c) 2022 Alwin Wang
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

import pyJanus


def test_Janus_return_type_policy():
    """
    This checks any return value policies that are not on the default policy
    `return_value_policy::automatic`

    Reference: <https://pybind11.readthedocs.io/en/stable/advanced/functions.html#return-value-policies>
    """
    xml_path = "Examples/CombinedExample.xml"
    janus = pyJanus.Janus(xml_path)

    angle_of_attack = janus.get_variabledef("angleOfAttack")

    assert id(janus.get_variabledef()[0]) == id(angle_of_attack)
    assert id(janus.get_variabledef("angleOfAttack")) == id(angle_of_attack)
    assert id(angle_of_attack.janus) == id(janus)


# Consider checking __repr__ and __str__


def test_JanusVariableManager_return_type_policy():
    # sourcery skip: equality-identity, use-assigned-variable
    """
    This checks any return value policies that are not on the default policy
    `return_value_policy::automatic`

    Reference: <https://pybind11.readthedocs.io/en/stable/advanced/functions.html#return-value-policies>
    """
    xml_path = "Examples/JanusVariableManagerExample.xml"
    jvm = pyJanus.JanusVariableManager(xml_path)

    input00 = pyJanus.JanusVariable(
        "input00", pyJanus.janusInputVariable, pyJanus.janusMandatory, "m s-1", 0.0
    )
    jInput00 = jvm.push_back(input00)
    jv_input00 = jvm[jInput00]

    assert id(jv_input00) == id(jvm[jInput00])



#
# pyJanus (Python Bindings for Janus)
#
# Copyright (c) 2022 Alwin Wang
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

import pyJanus


def test_mathml_example():
    """
    This checks pyJanus against the Examples/MathMLExample.cpp program
    """
    xml_path = "Examples/MathMLExample.xml"
    janus = pyJanus.Janus(xml_path)

    x = janus.get_variabledef("x")
    y = janus.get_variabledef("y")
    result = janus.get_variabledef("result")
    sign = janus.get_variabledef("sign")
    cosd = janus.get_variabledef("cosd")
    sind = janus.get_variabledef("sind")
    tand = janus.get_variabledef("tand")
    cscd = janus.get_variabledef("cscd")
    secd = janus.get_variabledef("secd")
    cotd = janus.get_variabledef("cotd")
    arccosd = janus.get_variabledef("arccosd")
    arcsind = janus.get_variabledef("arcsind")
    arctand = janus.get_variabledef("arctand")
    bound = janus.get_variabledef("bound")

    x.set_value(0.1)
    y.set_value(0.2)

    assert round(result.get_value(), 6) == 0.463648
    assert round(sign.get_value(), 1) == 0.1
    assert round(cosd.get_value(), 6) == 0.999998
    assert round(sind.get_value(), 8) == 0.00174533
    assert round(tand.get_value(), 8) == 0.00174533
    assert round(cscd.get_value(), 3) == 572.958
    assert round(secd.get_value(), 1) == 1.0
    assert round(cotd.get_value(), 3) == 572.957
    assert round(arccosd.get_value(), 4) == 84.2608
    assert round(arcsind.get_value(), 5) == 5.73917
    assert round(arctand.get_value(), 5) == 5.71059
    assert round(bound.get_value(), 1) == 0.5



# Adapted from https://github.com/pybind/cmake_example

#
# Copyright (c) 2016 The Pybind Development Team, All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# You are under no obligation whatsoever to provide any bug fixes, patches, or
# upgrades to the features, functionality or performance of the source code
# ("Enhancements") to anyone; however, if you choose to make your Enhancements
# available either publicly, or directly to the author of this software, without
# imposing a separate written license agreement for such Enhancements, then you
# hereby grant the following license: a non-exclusive, royalty-free perpetual
# license to install, use, modify, prepare derivative works, incorporate into
# other computer software, distribute, and sublicense such enhancements or
# derivative works thereof, in binary and source code form.
#

import os
import re
import subprocess
import sys

from setuptools import Extension, setup
from setuptools.command.build_ext import build_ext

# Convert distutils Windows platform specifiers to CMake -A arguments
PLAT_TO_CMAKE = {
    "win32": "Win32",
    "win-amd64": "x64",
    "win-arm32": "ARM",
    "win-arm64": "ARM64",
}


# A CMakeExtension needs a sourcedir instead of a file list.
# The name must be the _single_ output extension from the CMake build.
# If you need multiple extensions, see scikit-build.
class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=""):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def build_extension(self, ext):
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))

        # required for auto-detection & inclusion of auxiliary "native" libs
        if not extdir.endswith(os.path.sep):
            extdir += os.path.sep
        print(f"info: building {ext.name=} to {extdir=}")

        debug = int(os.environ.get("DEBUG", 0)) if self.debug is None else self.debug
        cfg = "Debug" if debug else "Release"
        print(f"info: building with {cfg=}")

        # CMake lets you override the generator - we need to check this.
        # Can be set with Conda-Build, for example.
        cmake_generator = os.environ.get("CMAKE_GENERATOR", "")

        # Set Python_EXECUTABLE instead if you use PYBIND11_FINDPYTHON
        cmake_args = [
            f"-DCMAKE_LIBRARY_OUTPUT_DIRECTORY={extdir}",
            f"-DPYTHON_EXECUTABLE={sys.executable}",
            f"-DCMAKE_BUILD_TYPE={cfg}",  # not used on MSVC, but no harm
            "-DBUILD_EXAMPLES=ON",
            "-DPYBIND11_BUILD_TESTS=ON",
            "-DBUILD_PYBINDINGS=ON",
        ]
        build_args = []
        # Adding CMake arguments set as environment variable
        # (needed e.g. to build for ARM OSx on conda-forge)
        if "CMAKE_ARGS" in os.environ:
            cmake_args += [item for item in os.environ["CMAKE_ARGS"].split(" ") if item]

        # In this example, we pass in the version to C++. You might not need to.
        cmake_args += [f"-DPY_VERSION_INFO={self.distribution.get_version()}"]

        if self.compiler.compiler_type == "msvc":

            # Single config generators are handled "normally"
            single_config = any(x in cmake_generator for x in {"NMake", "Ninja"})

            # CMake allows an arch-in-generator style for backward compatibility
            contains_arch = any(x in cmake_generator for x in {"ARM", "Win64"})

            # Specify the arch if using MSVC generator, but only if it doesn't
            # contain a backward-compatibility arch spec already in the
            # generator name.
            if not single_config and not contains_arch:
                cmake_args += ["-A", PLAT_TO_CMAKE[self.plat_name]]

            # Multi-config generators have a different way to specify configs
            if not single_config:
                cmake_args += [
                    f"-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{cfg.upper()}={extdir}"
                ]
                build_args += ["--config", cfg]

        elif not cmake_generator:
            # Using Ninja-build since it a) is available as a wheel and b)
            # multithreads automatically. MSVC would require all variables be
            # exported for Ninja to pick it up, which is a little tricky to do.
            # Users can override the generator with CMAKE_GENERATOR in CMake
            # 3.15+.
            try:
                import ninja  # noqa: F401

                cmake_args += ["-GNinja"]
            except ImportError:
                pass

        if sys.platform.startswith("darwin"):
            # Cross-compile support for macOS - respect ARCHFLAGS if set
            if archs := re.findall(r"-arch (\S+)", os.environ.get("ARCHFLAGS", "")):
                cmake_args += ["-DCMAKE_OSX_ARCHITECTURES={}".format(";".join(archs))]

        # Set CMAKE_BUILD_PARALLEL_LEVEL to control the parallel build level
        # across all generators.
        if (
            "CMAKE_BUILD_PARALLEL_LEVEL" not in os.environ
            and hasattr(self, "parallel")
            and self.parallel
        ):
            # self.parallel is a Python 3 only way to set parallel jobs by hand
            # using -j in the build_ext call, not supported by pip or PyPA-build.
            # CMake 3.12+ only.
            build_args += [f"-j{self.parallel}"]

        print(f"info: building with {cmake_args=}")

        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        print(f"info: building with {self.build_temp=}")

        subprocess.check_call(
            ["cmake", ext.sourcedir] + cmake_args, cwd=self.build_temp
        )
        subprocess.check_call(
            ["cmake", "--build", "."] + build_args, cwd=self.build_temp
        )


# The information here can also be placed in setup.cfg - better separation of
# logic and declaration, and simpler if you include description/version in a file.
setup(
    name="python-janus",
    version="0.1.0",
    author="Alwin Wang",
    author_email="16846521+AlwinW@users.noreply.github.com",
    description="python bindings for Janus",
    long_description="python bindings for Janus",
    url="https://github.com/alwinw/pyJanus",
    ext_modules=[CMakeExtension("pyJanus")],
    cmdclass={"build_ext": CMakeBuild},
    zip_safe=False,
    extras_require={"test": ["pytest>=6.0"]},
    python_requires=">=3.8",
)



#
# pyJanus (Python Bindings for Janus)
#
# Copyright (c) 2022 Alwin Wang
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

import pyJanus


def test_combined_example():
    """
    This checks pyJanus against the Examples/CombinedExample.cpp program
    """
    xml_path = "Examples/CombinedExample.xml"
    janus = pyJanus.Janus(xml_path)

    angle_of_attack = janus.get_variabledef("angleOfAttack")
    reynolds_number = janus.get_variabledef("reynoldsNumber")
    ambient_density = janus.get_variabledef("ambientDensity")
    true_airspeed = janus.get_variabledef("trueAirspeed")
    reference_area = janus.get_variabledef("referenceArea")
    drag = janus.get_variabledef("drag")

    angle_of_attack.set_value(10)
    reynolds_number.set_value(0.36e6)
    ambient_density.set_value(1.225)
    true_airspeed.set_value(100.0)
    reference_area.set_value(25.0)

    assert round(drag.get_value(), 2) == 1531.25
